@@ -5,6 +5,7 @@
 from __future__ import annotations
 
 import re
+from contextlib import suppress
 from enum import Enum, auto
 from logging import getLogger
 from os import getenv
@@ -167,11 +168,32 @@ class ADBProcess:
             if self.logs and (self.logs / "log_asan.txt").is_file():
                 self.reason = Reason.ALERT
         except ADBSessionError:
-            LOG.warning("No device detected while closing process")
-        self._pid = None
-        self.profile = None
-        if self.reason is None:
-            self.reason = Reason.CLOSED
+            LOG.error("No device detected while closing process")
+        finally:
+            if self.reason is None:
+                self.reason = Reason.CLOSED
+            self.profile = None
+            self._pid = None
+
+    def cpu_usage(self) -> Generator[tuple[int, float]]:
+        """Collect percentage of CPU usage per package process.
+
+        Args:
+            None
+
+        Yields:
+            PID and the CPU usage as a percentage.
+        """
+        result = self._session.shell(
+            ("top", "-b", "-n", "1", "-m", "30", "-q", "-o", "PID,%CPU,CMDLINE"),
+            device_required=False,
+        )
+        if result.exit_code == 0:
+            for entry in result.output.splitlines():
+                pid, cpu_pct, args = entry.lstrip().split(maxsplit=2)
+                if self._package in args:
+                    with suppress(ValueError):
+                        yield int(pid), float(cpu_pct)
 
     def find_crashreports(self) -> Generator[PurePosixPath]:
         """Scan for crash reports.
@@ -244,7 +266,7 @@ class ADBProcess:
         Returns:
             True if the browser was successfully launched otherwise False.
         """
-        LOG.debug("launching - url: %s", url)
+        LOG.debug("launching (%ds): %s", launch_timeout, url)
         assert self._launches > -1, "clean_up() has been called"
         assert self._pid is None, "Process is already running"
         assert self.reason is not None, "Process is already running"
@@ -348,6 +370,8 @@ class ADBProcess:
                 raise ADBLaunchError(f"Could not launch '{self._package}'")
             self._pid = self._session.get_pid(self._package)
             bootstrapper.wait(self.is_healthy, url=url)
+            # prevent power management and backgrounding
+            self._session.shell(["am", "set-inactive", self._package, "false"])
         finally:
             self._session.reverse_remove(bootstrapper.port)
             bootstrapper.close()
@@ -429,9 +453,6 @@ class ADBProcess:
             None
         """
         assert self.logs is None
-        if self.profile is None:
-            LOG.debug("no logs to process since browser was not launched")
-            return
         # TODO: use a common tmp dir
         self.logs = Path(mkdtemp(prefix="mp-logs_"))
 
@@ -491,39 +512,27 @@ class ADBProcess:
         if not logcat.is_file():
             LOG.warning("log_logcat.txt does not exist!")
             return
-        err_log = logs / "log_stderr.txt"
-        if err_log.is_file():
-            LOG.warning("log_stderr.txt already exist! Overwriting...")
-        out_log = logs / "log_stdout.txt"
-        if out_log.is_file():
-            LOG.warning("log_stdout.txt already exist! Overwriting...")
-        package_name_bytes = package_name.encode("utf-8")
         # create set of filter pids
         # this will include any line that mentions "Gecko", "MOZ_" or the package name
+        tokens = (b"Gecko", b"MOZ_", b"wrap.sh", package_name.encode("utf-8"))
         asan_tid = None
         filter_pids = set()
         re_id = re.compile(rb"^\d+-\d+\s+(\d+[:.]){3}\d+\s+(?P<pid>\d+)\s+(?P<tid>\d+)")
         with logcat.open("rb") as lc_fp:
             for line in lc_fp:
-                if (
-                    b"Gecko" not in line
-                    and b"MOZ_" not in line
-                    and package_name_bytes not in line
-                    and b"wrap.sh" not in line
-                ):
+                if all(x not in line for x in tokens):
                     continue
                 m_id = re_id.match(line)
-                if m_id is None:
-                    continue
-                filter_pids.add(m_id.group("pid"))
-                if asan_tid is None and b": AddressSanitizer:" in line:
-                    asan_tid = m_id.group("tid")
+                if m_id is not None:
+                    filter_pids.add(m_id.group("pid"))
+                    if asan_tid is None and b": AddressSanitizer:" in line:
+                        asan_tid = m_id.group("tid")
         LOG.debug("%d interesting pid(s) found in logcat output", len(filter_pids))
         # filter logs
         with (
             logcat.open("rb") as lc_fp,
-            err_log.open("wb") as e_fp,
-            out_log.open("wb") as o_fp,
+            (logs / "log_stderr.txt").open("wb") as e_fp,
+            (logs / "log_stdout.txt").open("wb") as o_fp,
         ):
             for line in lc_fp:
                 # quick check if pid is in the line
@@ -637,14 +646,20 @@ class ADBProcess:
         # TODO: is this the best way???
         self._session.shell(["am", "force-stop", self._package])
 
-    def wait(self) -> None:
-        """Wait for browser process.
+    def wait(self, timeout: float | None = None) -> bool:
+        """Wait for process to terminate. If a timeout of zero or greater is specified
+        the call will block until the timeout expires.
 
         Args:
-            None
+            timeout: The maximum amount of time in seconds to wait or
+                     None (wait indefinitely).
 
-        Return:
-            None
+        Returns:
+            True if the process exits before the timeout expires otherwise False.
         """
+        deadline = None if timeout is None else time() + timeout
         while self.is_running():
+            if deadline is not None and deadline <= time():
+                return False
             sleep(0.25)
+        return True
